<?php
namespace App\Security;

use App\Kernel;
use App\Entity\User;
use App\Repository\UserRepository;
use Psr\Log\LoggerInterface;
use Symfony\Component\Security\Core\Exception\BadCredentialsException;
use Symfony\Component\Security\Http\AccessToken\AccessTokenHandlerInterface;
use Symfony\Component\Security\Http\Authenticator\Passport\Badge\UserBadge;
use DateInterval;
use DateTimeImmutable;
use DateTimeZone;

class AccessTokenHandler implements AccessTokenHandlerInterface
{

    public function __construct(private Kernel $kernel, private UserRepository $userRepository,
        private LoggerInterface $logger)
    {}

    public function getUserBadgeFrom(string $accessToken): UserBadge
    {
        if ($accessToken === FALSE) {
            throw new BadCredentialsException('Missing credentials.');
        }

        $this->logger->info("Environment is " . $this->kernel->getEnvironment());

        $validity = "8 hour";
        if ("dev" == $this->kernel->getEnvironment()) {
            $validity = "1 year";
        }

        $tokenKeys = array(
            "AUTH_SEED" => $validity
        );

        foreach ($tokenKeys as $key => $value) {
            $authName = $this->parseMessage($accessToken, $key, $value);

            if ($authName !== NULL) {
                return new UserBadge($authName, fn (string $id) => $this->getOrCreateUser($id));
            }
        }

        throw new BadCredentialsException('Invalid credentials.');
    }

    private function getOrCreateUser(string $authName): User
    {
        $user = $this->userRepository->findOneByAuthName($authName);
        if (NULL === $user) {
            $user = new User($authName);
        }
        return $user;
    }

    public function parseMessage(string $accessToken, string $seedKey, string $duration): ?string
    {
        $sign_seed = sodium_base642bin($_ENV[$seedKey], SODIUM_BASE64_VARIANT_ORIGINAL);
        $sign_pair = sodium_crypto_sign_seed_keypair($sign_seed);
        $sign_public = sodium_crypto_sign_publickey($sign_pair);
        $message_signed = sodium_base642bin($accessToken, SODIUM_BASE64_VARIANT_URLSAFE);
        $message = sodium_crypto_sign_open($message_signed, $sign_public);

        if ($message === FALSE) {
            return NULL;
        }

        $arr = explode('|', $message);

        $ts = new DateTimeImmutable($arr[1], new DateTimeZone("UTC"));
        $now = new DateTimeImmutable("now", new DateTimeZone("UTC"));

        $ts = $ts->add(DateInterval::createFromDateString($duration));
        if ($ts < $now) {
            throw new BadCredentialsException('Token has already expired.');
        }

        return $arr[0];
    }

    public function encodeMessage(string $authName, string $seedKey): string
    {
        $now = new DateTimeImmutable("now", new DateTimeZone("UTC"));

        $sign_seed = sodium_base642bin($_ENV[$seedKey], SODIUM_BASE64_VARIANT_ORIGINAL);
        $sign_pair = sodium_crypto_sign_seed_keypair($sign_seed);
        $sign_secret = sodium_crypto_sign_secretkey($sign_pair);
        $message = sodium_crypto_sign($authName . '|' . $now->format("c"), $sign_secret);

        return sodium_bin2base64($message, SODIUM_BASE64_VARIANT_URLSAFE);
    }
}

